# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

__all__ = ['Distribution']

import os
from pkg_resources import (
  get_distribution, DistributionNotFound, VersionConflict)


class Distribution(str):
    """Simple proxy to get a pkg_resources.Distribution instance
       matching the given name and :class:`Version` instance.
    """
    def __new__(cls, name, pkg, version):
        return str.__new__(cls, name)

    def __init__(self, name, pkg, version):
        self.pkg = pkg
        self.version = version

    def find(self, modpath, raise_=True):
        """Try to find the distribution and check version.

        :param raise_: Raise a VersionConflict
          if version doesn't match the given one?
          If false just return None.
        """
        # If no version is given (for whatever reason), just do nothing:
        if not self.version:
            return None
        try:
            dist = get_distribution(self)
        except DistributionNotFound:
            return None
        if not os.path.samefile(
          dist.location, os.path.realpath(os.path.dirname(modpath))
          ):
            return None
        if dist.parsed_version != self.version.parsed:
            if raise_:
                raise VersionConflict(
                  "Version of distribution %s"
                  " doesn't match %s.__version__ %s."
                  % (dist, self.pkg, self.version))
            return None
        return dist



# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

__all__ = ['Zetup', 'find_zetup_config']

import sys
import os
from inspect import ismodule

from .zetup import Zetup


def find_zetup_config(pkgname):
    zfg_pkgname = pkgname + '.zetup_config'
    try: # Already imported?
        return sys.modules[zfg_pkgname]
    except KeyError:
        pass
    try:
        return __import__(zfg_pkgname).zetup_config
    except ImportError:
        pass
    # ==> no .zetup_config subpackage
    # ==> assume package imported from source (repo)
    # ==> load setup config from package's parent path:
    mod = sys.modules[pkgname]
    path = os.path.dirname(os.path.dirname(os.path.realpath(mod.__file__)))
    return Zetup(path)


# Get zetup's own config:
zfg = find_zetup_config(__name__)

__distribution__ = zfg.DISTRIBUTION.find(__path__[0])
__description__ = zfg.DESCRIPTION

__version__ = zfg.VERSION

__requires__ = zfg.REQUIRES # .checked
if ismodule(zfg): # if this is an installed zetup package:
    __requires__.check()

__extras__ = zfg.EXTRAS

__notebook__ = zfg.NOTEBOOKS['README']


def setup_entry_point(dist, keyword, value):
    """zetup's `entry_point` handler for `setup()` in a project's _setup.py_,
       setting all setup keyword parameters based on zetup config.

    - Activated with `setup(setup_requires=['zetup'], use_setup=True)`
    """
    assert keyword == 'use_zetup'
    if not value:
        return

    zetup = Zetup() # reads config from current working directory
                    #  (where setup.py is run)
    keywords = zetup.setup_keywords()
    for name, value in keywords.items():
        # Generally, setting stuff on dist.metadata is enough
        #  (and necessary), but *pip* only works correct
        #  if stuff is also set directly on dist object
        #  (seems to read at least packages list somehow from there):
        for obj in [dist, dist.metadata]:
            setattr(obj, name, value)



# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

__all__ = ['init', 'ZetupCommandError', 'ZetupMakeError']

import os

import zetup

__requires__ = zetup.__extras__['commands'].checked

from path import path as Path


class ZetupCommandError(RuntimeError):
    pass


def init(name, path=None):
    path = Path(path or os.getcwd())
    Path(__path__[0] / 'zetup.py').copy(path / '__init__.py')
    with open(path / 'zetuprc', 'w') as f:
        f.write("[%s]\n\n%s\n" % (name, "\n".join("%s =" % key for key in [
          'description',
          'author',
          'url',
          'license',
          'python',
          'classifiers',
          'keywords',
          ])))
    with open(path / 'VERSION', 'w') as f:
        f.write("0.0.0\n")


from . import make, run, pytest, tox, conda
from .make import ZetupMakeError



# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

__all__ = ['Zetup']

try:
    from setuptools import setup, Command
except ImportError: # fallback
    # (setuptools should at least be available after package installation)
    from distutils.core import setup, Command

from .config import load_zetup_config


class Zetup(object):
    def __init__(self, ZETUP_DIR='.'):
        """Load and store zetup config from `ZETUP_DIR`
           as attributes in `self`.
        """
        load_zetup_config(ZETUP_DIR, cfg=self)

    def setup_keywords(self):
        """Get a dictionary of `setup()` keywords generated from zetup config.
        """
        keywords = {
          'name': self.NAME,
          'description': self.DESCRIPTION,
          'author': self.AUTHOR,
          'author_email': self.EMAIL,
          'url': self.URL,
          'license': self.LICENSE,
          'install_requires': str(self.REQUIRES),
          'extras_require':
            {name: str(reqs) for name, reqs in self.EXTRAS.items()},
          'classifiers': self.CLASSIFIERS,
          'keywords': self.KEYWORDS,
          }
        if self.VERSION:
          keywords['version'] = str(self.VERSION)
        if self.PACKAGES:
            keywords['packages'] = self.PACKAGES
        if self.ZETUP_CONFIG_PACKAGE:
            keywords.update({
              'package_dir': {self.ZETUP_CONFIG_PACKAGE: '.'},
              'package_data': {self.ZETUP_CONFIG_PACKAGE: self.ZETUP_DATA},
              })
        cmdclasses = {}
        for cmdname in self.COMMANDS:
            cmdmethod = getattr(self, cmdname)

            class ZetupCommand(Command):
                # Must override options handling stuff from Command base...
                user_options = []

                def initialize_options(self):
                    pass

                def finalize_options(self):
                    pass

                run = staticmethod(cmdmethod)

            ZetupCommand.__name__ = cmdname
            cmdclasses[cmdname] = ZetupCommand

        keywords['cmdclass'] = cmdclasses
        return keywords

    @property
    def setup(self):
        return Setup(zetup=self)

    def __call__(self, **setup_keywords):
        """Run `setup()` with generated keywords from zetup config
           and custom override `setup_keywords`.
        """
        return self.setup(**setup_keywords)

    COMMANDS = []

    @classmethod
    def command(cls, args=None, depends=None):
        return CommandDeco(cls, args, depends)


class Setup(dict):
    def __init__(self, zetup):
        dict.__init__(self, zetup.setup_keywords())
        self.zetup = zetup

    def __call__(self, **keywords):
        """Run `setup()` with generated keywords from zetup config
           and custom override `keywords`.
        """
        keywords = dict(self, **keywords)
        if 'make' in Zetup.COMMANDS:
            with self.zetup.make(
              targets=['VERSION', 'setup.py', '__init__.py'],
              skip_existing=True
              ):
                return setup(**keywords)
        return setup(**keywords)


class CommandDeco(object):
    def __init__(self, zetupcls, args=None, depends=None):
        self.zetupcls = zetupcls
        self.args = args
        self.make_targets = depends and list(depends)

    def __call__(self, func):
        """Add a command function as method to :class:`Zetup`
           and store its name in `cls.COMMANDS`.

        - Used in `zetup.commands.*` to sparate command implementations.
        """
        targets = self.make_targets

        def cmdmethod(self, args=None, **kwargs):
            if not targets:
                return func(self, args, **kwargs)
            with self.make(targets=targets, skip_existing=True):
                return func(self, args, **kwargs)

        name = cmdmethod.__name__ = func.__name__
        cmdmethod.args = self.args

        setattr(self.zetupcls, name, cmdmethod)
        self.zetupcls.COMMANDS.append(name)


# If installed with pip, add all build directories and src/ subdirs
#  of implicitly downloaded requirements
#  to sys.path and os.environ['PYTHONPATH']
#  to make them importable during installation:
# sysbuildpath = os.path.join(sys.prefix, 'build')
# try:
#     fnames = os.listdir(sysbuildpath)
# except OSError:
#     pass
# else:
#     if 'pip-delete-this-directory.txt' in fnames:
#         pkgpaths = []
#         for fn in fnames:
#             path = os.path.join(sysbuildpath, fn)
#             if not os.path.isdir(path):
#                 continue
#             path = os.path.abspath(path)
#             pkgpaths.append(path)

#             srcpath = os.path.join(path, 'src')
#             if os.path.isdir(srcpath):
#                 pkgpaths.append(srcpath)

#         for path in pkgpaths:
#             sys.path.insert(0, path)

#         PYTHONPATH = os.environ.get('PYTHONPATH')
#         PATH = ':'.join(pkgpaths)
#         if PYTHONPATH is None:
#             os.environ['PYTHONPATH'] = PATH
#         else:
#             os.environ['PYTHONPATH'] = ':'.join([PATH, PYTHONPATH])


# If this is a locally imported zetup.py in zetup's own repo...
# if (NAME == 'zetup' #==> is in zetup's own package/repo
#     and os.path.basename(__file__).startswith('zetup')
#     #"=> was not exec()'d from setup.py
#     and not 'zetup.zetup' in sys.modules
#     #"=> was not imported as subpackage
#     ):
#     #... then fake the interface of the installed zetup package...
#     import zetup # import itself as faked subpackage

#     ## __path__ = [os.path.join(ZETUP_DIR, 'zetup')]
#     __path__ = [ZETUP_DIR]
#     # Exec the __init__ which gets installed in top level zetup package:
#     exec(open('__init__.py').read().replace('from . import zetup', ''))



# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

import os
from textwrap import dedent

from path import path as Path

from jinja2 import FileSystemLoader, TemplateNotFound
from jinjatools import Environment

from zetup import Zetup
from zetup.commands import ZetupCommandError


class Loader(FileSystemLoader):
    def __init__(self):
        self.templates_dir = Path(__path__[0]) / 'templates'
        super(Loader, self).__init__(self.templates_dir)

    def get_source(self, env, target):
        source, _, uptodate = super(Loader, self).get_source(
          env, target + '.jinja')
        return source, target, uptodate


class Made(list):
    def __init__(self):
        self.status = 0

    def clean(self):
        for path in self:
            path.remove()

    def __enter__(self):
        return self

    def __exit__(self, *exc_info):
        self.clean()


class ZetupMakeError(ZetupCommandError):
    def __init__(self, made, message):
        ZetupCommandError.__init__(self, message)
        made.clean()


@Zetup.command()
def make(self, args=None, targets=None, force=False, skip_existing=False):
    if args:
        targets = args.targets
        force = args.force
    if not targets:
        raise ZetupCommandError("No targets given. You can 'make all'.")
    env = Environment(loader=Loader())
    if 'all' in targets:
        targets = [tname.rsplit('.', 1)[0]
                   for tname in os.listdir(env.loader.templates_dir)
                   if tname.endswith('.jinja')]
        skip_existing = True

    made = Made()
    for target in targets:
        path = Path(self.ZETUP_DIR) / target
        if path.exists():
            if skip_existing:
                continue
            if not force:
                raise ZetupMakeError(made,
                  "Target '%s' already exists." % target
                  + "Overwrite with -f or --force")
        try:
            template = env.get_template(target)
        except TemplateNotFound:
            raise ZetupMakeError(made,
              "No template for target '%s'." % target)
        text = template.render({
          'zetup_header': dedent("""
            # This file was auto-generated by zetup
            #
            # https://bitbucket.org/userzimmermann/zetup.py
            """),
          'zfg': self,
          })
        path.write_text(text.strip())
        made.append(path)

    return made



# This file was auto-generated by zetup
#
# https://bitbucket.org/userzimmermann/zetup.py


import sys

from zetup.config import load_zetup_config



load_zetup_config(__path__[0], sys.modules[__name__])


# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

from subprocess import call

from zetup import Zetup


@Zetup.command(depends=['all'])
def run(self, args=None, cmd=None):
    if args:
        cmd = args.cmd
    return call(cmd)



# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

__all__ = ['Requirements']

import sys
if sys.version_info[0] == 3:
    unicode = str

from pkg_resources import (
  parse_requirements, Requirement,
  get_distribution, DistributionNotFound, VersionConflict)


class Requirements(str):
    """Package requirements manager.
    """
    @staticmethod
    def _parse(text):
        """ Generate parsed requirements from `text`,
            which should contain newline separated requirement specs.

        - Additionally looks for "#import name" comments
          after requirement lines (the actual root module name
          of the required package to use for runtime dependency checks)
          and stores them as .impname attrs on the Requirement instances.
        """
        for line in text.split('\n'):
            line = line.strip()
            if not line:
                continue
            try:
                req, impname = line.split('#import')
            except ValueError:
                req = next(parse_requirements(line))
                req.impname = req.key
            else:
                req = next(parse_requirements(req))
                req.impname = impname.strip()
            yield req

    def __new__(cls, reqs):
        """Store a list of :class:`pkg_resources.Requirement` instances
           from the given requirement specs
           and additionally store them newline separated
           in the :class:`str` base.

        :param reqs: Either a single string of requirement specs
          or a sequence of strings and/or
          :class:`pkg_resources.Requirement` instances.
        """
        if isinstance(reqs, (str, unicode)):
            reqlist = list(cls._parse(reqs))
        else:
            reqlist = []
            for req in reqs:
                if isinstance(req, (str, unicode)):
                    reqlist.extend(cls._parse(req))
                elif isinstance(req, Requirement):
                    reqlist.append(req)
                else:
                    raise TypeError(type(req))

        obj = str.__new__(cls, '\n'.join(map(str, reqlist)))
        obj._list = reqlist
        return obj

    def check(self, raise_=True):
        """Check that all requirements are available (importable)
           and their versions match (using modules' __version__ attributes).

        - Fallback to pkg_resources.get_distribution().version
          if no __version__ or is None.

        :param raise_: Raise DistributionNotFound if ImportError
          or VersionConflict if version doesn't match?
          If False just return False in that case.
        """
        for req in self:
            try:
                mod = __import__(req.impname)
            except ImportError:
                if raise_:
                    raise DistributionNotFound(str(req))
                return False
            if not req.specs: # No version constraints
                continue
            try:
                version = mod.__version__
                if version is None:
                    raise AttributeError(
                      "module's '__version__' attribute is None")
            except AttributeError as e_no__version__attr:
                try: # try to get version from distribution
                    dist = get_distribution(req.key)
                except DistributionNotFound as e:
                    if raise_:
                        raise VersionConflict("Need %s. %s: %s. %s: %s" % (
                          req, e_no__version__attr, mod,
                          type(e).__name__, e))
                    return False
                version = dist.version
            if version not in req:
                if raise_:
                    raise VersionConflict(
                      "Need %s. Found %s" % (req, version))
                return False
        return True

    @property
    def checked(self):
        self.check()
        return self

    def __iter__(self):
        return iter(self._list)

    def __getitem__(self, name):
        """Get a requirement by its distribution name.
        """
        for req in self._list:
            if name in [req.key, req.unsafe_name]:
                return req
        raise KeyError(name)

    def __add__(self, text):
        """Return a new :class:`Requirements` instance
           with additional requirements from `text`.
        """
        return type(self)('%s\n%s' % (
          # For simplicity:
          #  Just create explicit #import hints for every requirement:
          '\n'.join('%s #import %s' % (req, req.impname) for req in self),
          text))

    def __repr__(self):
        return str(self)



# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

__all__ = ['Extras']

from itertools import chain
from collections import OrderedDict

from .requires import Requirements


class Extras(OrderedDict):
    """Package extra features/requirements manager.

    - Stores :class:`Requirements` instances by extra feature name keys.
    - Provides an implicit 'all' key, returning a dynamically created
      combined :class:`Requirements` instance with all extra requirements.
    """
    def __setitem__(self, name, text):
        reqs = Requirements(text)
        OrderedDict.__setitem__(self, name, reqs)

    def __getitem__(self, name):
        if name == 'all':
            return Requirements(chain(*self.values()))
        return OrderedDict.__getitem__(self, name)

    def __repr__(self):
        return "\n\n".join((
          "[%s]\n" % key + '\n'.join(map(str, reqs))
            for key, reqs in self.items() if key != 'all'))



from inspect import ismodule

from pkg_resources import get_distribution

from path import path as Path

from pytest import raises

import zetup
from zetup import Zetup, find_zetup_config


def test_zetup():
    zfg = find_zetup_config('zetup')
    assert zfg.NAME == 'zetup'

    pkg_path = Path(zetup.__path__[0]).realpath()
    zfg_path = Path(zfg.ZETUP_DIR).realpath()

    in_site_packages = ismodule(zfg)
    in_repo = not in_site_packages

    if in_site_packages:
        assert zfg.__name__ == 'zetup.zetup_config'
        assert zfg_path == pkg_path / 'zetup_config'
    else:
        with raises(ImportError):
            from zetup import zetup_config
        assert isinstance(zfg, Zetup)
        assert zfg_path == pkg_path.dirname()

    assert zetup.__description__ == zfg.DESCRIPTION

    dist = get_distribution('zetup')
    if in_repo and not Path(dist.location).samefile(pkg_path.dirname()):
        dist = None
    assert zetup.__distribution__ == zfg.DISTRIBUTION.find(pkg_path) \
      == dist

    assert zetup.__version__ == zfg.VERSION
    if in_site_packages:
        assert zetup.__version__ == (zfg_path / 'VERSION').text().strip()

    assert str(zetup.__requires__) == str(zfg.REQUIRES)
           ## == (path / 'requirements.txt').text())



# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

__all__ = ['Notebook']

import re


try:
    from path import path as Path
except ImportError:
    class Notebook(str):
        pass
else:
    try:
        from IPython import nbconvert
        from jinja2 import BaseLoader, TemplateNotFound
    except ImportError:
        class Notebook(Path):
            pass
    else:
        from inspect import getmembers
        from textwrap import dedent


        def bitbucket_rst_links(rst):
            return re.sub(
              r'<#[0-9.-]+([^>]+)>',
              lambda match: '<#rst-header-%s>' % match.group(1).lower(),
              rst)


        def github_markdown_links(markdown):
            return re.sub(
              r'\[([^\]]+)\]: #(.+)',
              lambda match: '[%s]: #%s' % (
                match.group(1), match.group(2).replace('.', '').lower()),
                markdown)


        class ExtraTemplateLoader(BaseLoader):
            templates = {
              'bitbucket_rst': dedent("""
                {%- extends 'rst.tpl' -%}


                {% block input %}
                {%- if cell.input.strip() -%}

                .. code:: python

                {% if cell.outputs %}
                {{ cell.input | add_prompts(cont='>>> ') | indent }}
                {% else %}
                {{ cell.input | indent }}
                {% endif %}
                {%- endif -%}
                {% endblock input %}


                {% block stream %}
                {{ output.text | indent }}
                {% endblock stream %}


                {% block data_text scoped %}
                {{ output.text | indent }}
                {% endblock data_text %}


                {% block markdowncell scoped %}
                {{ super() | bitbucket_rst_links }}
                {% endblock markdowncell %}
                """),

              'github_markdown': dedent("""
                {%- extends 'markdown.tpl' -%}


                {% block input %}
                ```python
                {% if cell.outputs %}
                {{ cell.input | add_prompts(cont='>>> ') }}
                {% else %}
                {{ cell.input }}
                ```
                {% endif %}
                {% endblock input %}


                {% block stream %}
                {{ output.text }}
                ```
                {% endblock stream %}


                {% block data_text scoped %}
                {{ output.text }}
                ```
                {% endblock data_text %}


                {% block markdowncell scoped %}
                {{ super() | github_markdown_links }}
                {% endblock markdowncell %}
                """),
              }

            def get_source(self, env, template):
                try:
                    return self.templates[template], template, True
                except KeyError:
                    raise TemplateNotFound(template)


        class Notebook(Path):
            EXTRA_FILTERS = {
              'bitbucket_rst_links': bitbucket_rst_links,
              'github_markdown_links': github_markdown_links,
              }

            EXTRA_LOADER = ExtraTemplateLoader()

            def to_bitbucket_rst(self):
                """Export the notebook to Bitbucket optimized reST.

                - Prepends 'rst-header-' to link targets.
                  ('#some-section' --> '#rst-header-some-section')
                """
                rst = nbconvert.export_rst(self,
                  filters=self.EXTRA_FILTERS,
                  extra_loaders=[self.EXTRA_LOADER],
                  # Not a real file,
                  #  will be loaded from string by EXTRA_LOADER:
                  template_file='bitbucket_rst',
                  )[0]
                # bitbucket_rst template puts code cell input and output
                #  in single blocks, but can't prevent empty lines in between
                #==> Remove them:
                return re.sub(
                  # .. code:: python
                  #
                  #     >>> input
                  #
                  #     output
                  r'(\n    >>> .+\n)\s*\n(    [^>])',
                  # .. code:: python
                  #
                  #     >>> input
                  #     output
                  r'\1\2',
                  rst)

            def to_github_markdown(self):
                """Export the notebook to Github optimized Markdown.

                - Removes header enumeration prefixes from link targets.
                  ('#1.-section-one' --> '#section-one')
                """
                markdown = nbconvert.export_markdown(self,
                  filters=self.EXTRA_FILTERS,
                  extra_loaders=[self.EXTRA_LOADER],
                  # Not a real file,
                  #  will be loaded from string by EXTRA_LOADER:
                  template_file='github_markdown',
                  )[0]
                # github_markdown template puts code cell input and output
                #  in single blocks, but can't prevent empty lines in between
                #==> Remove them:
                markdown = re.sub(
                  # ```python
                  # >>> input
                  #
                  # output
                  # ```
                  r'(\n>>> .+\n)\s*\n([^>`])',
                  # ```python
                  # >>> input
                  # output
                  # ```
                  r'\1\2',
                  markdown)
                # And also remove newlines after ```python and before ```:
                markdown = re.sub(r'(\n```python.*\n)\s*', r'\1', markdown)
                markdown = re.sub(r'\n\s*(```\s*\n)', r'\n\1', markdown)
                return markdown


        def _exportmethod(name, func):
            def method(self):
                return func(self)

            method.__name__ = 'to_' + name
            return method


        for name, member in getmembers(nbconvert):
            if name.startswith('export_'):
                name = name.split('_', 1)[1]
                if name != 'by_name':
                    setattr(Notebook, 'to_' + name,
                            _exportmethod(name, member))



from textwrap import dedent
from warnings import warn

from zetup import Zetup
try: # needs zetup's requirements to be installed
    import zetup.commands
except Exception as e:
    warn(dedent("""
      Can't load all zetup features.
      You should install requirements from 'requirements.commands.txt'
      (ImportError: %s)
      """ % e).strip())


zetup = Zetup()

setup = zetup.setup
setup['package_data']['zetup.commands.make'] = ['templates/*.jinja']
setup(
  setup_requires=['hgdistver'],

  get_version_from_scm=True,

  entry_points={'distutils.setup_keywords': [
    'use_zetup = zetup:setup_entry_point',
    ]},

  scripts=['scripts/zetup'],
  )



# zetup.py
#
# Zimmermann's Python package setup.
#
# Copyright (C) 2014 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# zetup.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# zetup.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with zetup.py. If not, see <http://www.gnu.org/licenses/>.

__all__ = ['load_zetup_config']

import sys
import os
import re
from itertools import chain
from collections import OrderedDict
if sys.version_info[0] == 3:
    from configparser import ConfigParser
    # Just for simpler PY2/3 compatible code:
    unicode = str
else:
    from ConfigParser import ConfigParser

from setuptools import find_packages

from .version import Version
from .requires import Requirements
from .extras import Extras
from .dist import Distribution
from .notebook import Notebook


def load_zetup_config(path, cfg):
    """Load zetup config from directory in `path`
       and store keywords as attributes to `cfg` object.
    """
    cfg.ZETUP_DIR = path

    # Read the zetup config...
    config = ConfigParser()
    for fname in ['zetup.ini', 'zetup.cfg', 'zetuprc']:
        cfg.ZETUP_FILE = os.path.join(cfg.ZETUP_DIR, fname)
        if config.read(cfg.ZETUP_FILE):
            ##TODO: No print if run from installed package (under pkg/zetup/):
            ## print("zetup: Using config from %s" % fname)

            # The config file will be installed as pkg.zetup package_data:
            cfg.ZETUP_DATA = [fname]
            break
    else:
        raise RuntimeError("No zetup config found.")

    #... and store all setup options in UPPERCASE vars...
    cfg.NAME = config.sections()[0]

    config = dict(config.items(cfg.NAME))

    cfg.TITLE = config.get('title', cfg.NAME)
    cfg.DESCRIPTION = config['description'].strip().replace('\n', ' ')

    cfg.AUTHOR = re.match(r'^([^<]+)<([^>]+)>$', config['author'])
    cfg.AUTHOR, cfg.EMAIL = map(str.strip, cfg.AUTHOR.groups())
    cfg.URL = config['url']

    cfg.LICENSE = config['license']

    cfg.PYTHON = config['python'].split()

    cfg.PACKAGES = config.get('packages', [])
    if cfg.PACKAGES:
        # First should be the root package
        cfg.PACKAGES = cfg.PACKAGES.split()
    elif os.path.isdir(os.path.join(cfg.ZETUP_DIR, cfg.NAME)):
        # Just assume distribution name == root package name
        cfg.PACKAGES = [cfg.NAME]

    cfg.ZETUP_CONFIG_PACKAGE = config.get(
      'zetup_config_package', False)
    if cfg.ZETUP_CONFIG_PACKAGE:
        if cfg.ZETUP_CONFIG_PACKAGE == 'yes':
            cfg.ZETUP_CONFIG_PACKAGE = cfg.PACKAGES[0] + '.zetup_config'
        elif cfg.ZETUP_CONFIG_PACKAGE != 'no':
            cfg.ZETUP_CONFIG_PACKAGE = cfg.ZETUP_CONFIG_PACKAGE.strip()

    cfg.CLASSIFIERS = config['classifiers'].strip() \
      .replace('\n::', ' ::').split('\n')
    cfg.CLASSIFIERS.append('Programming Language :: Python')
    for pyversion in cfg.PYTHON:
        cfg.CLASSIFIERS.append('Programming Language :: Python :: ' + pyversion)

    cfg.KEYWORDS = config['keywords'].split()
    if any(pyversion.startswith('3') for pyversion in cfg.PYTHON):
        cfg.KEYWORDS.append('python3')

    # The default pkg.zetup package for installing this script and ZETUP_DATA:
    if cfg.PACKAGES:
        cfg.ZETUP_PACKAGE = cfg.PACKAGES[0] + '.zetup'

    # Extend PACKAGES with all their subpackages:
    try:
        find_packages
    except NameError: #==> No setuptools
        pass
    else:
        cfg.PACKAGES.extend(chain(*(
          ['%s.%s' % (pkg, sub) for sub in find_packages(pkg)]
          for pkg in cfg.PACKAGES if os.path.isdir(pkg))))

    if cfg.ZETUP_CONFIG_PACKAGE:
        cfg.PACKAGES.append(cfg.ZETUP_CONFIG_PACKAGE)

    # Parse VERSION and requirements files
    #  and add them to pkg.zetup package_data...
    cfg.ZETUP_DATA += ['VERSION', 'requirements.txt']

    cfg.VERSION_FILE = os.path.join(cfg.ZETUP_DIR, 'VERSION')
    if os.path.exists(cfg.VERSION_FILE):
        cfg.VERSION = open(cfg.VERSION_FILE).read().strip()
    else:
        cfg.VERSION_FILE = None
        try:
            import hgdistver
        except ImportError:
            cfg.VERSION = None
        else:
            cfg.VERSION = hgdistver.get_version()
    if cfg.VERSION:
        cfg.VERSION = Version(cfg.VERSION)

    cfg.DISTRIBUTION = Distribution(
      cfg.NAME, cfg.PACKAGES and cfg.PACKAGES[0] or cfg.NAME, cfg.VERSION)

    cfg.REQUIRES = Requirements(
      open(os.path.join(cfg.ZETUP_DIR, 'requirements.txt')).read())

    # Look for optional extra requirements to use with setup's extras_require=
    cfg.EXTRAS = Extras()
    _re = re.compile(r'^requirements\.(?P<name>[^\.]+)\.txt$')
    for fname in sorted(os.listdir(cfg.ZETUP_DIR)):
        match = _re.match(fname)
        if match:
            cfg.ZETUP_DATA.append(fname)

            cfg.EXTRAS[match.group('name')] \
              = open(os.path.join(cfg.ZETUP_DIR, fname)).read()

    # Are there IPython notebooks?
    cfg.NOTEBOOKS = OrderedDict()
    for fname in sorted(os.listdir(cfg.ZETUP_DIR)):
        name, ext = os.path.splitext(fname)
        if ext == '.ipynb':
            if name == 'README':
                cfg.ZETUP_DATA.append(fname)
            cfg.NOTEBOOKS[name] = Notebook(
              os.path.join(cfg.ZETUP_DIR, fname))
